import palang

JsonList = palang.import_model("hello::JsonList")
YamlList = palang.import_model("hello::YamlList")

tell_joke = palang.import_program("hello::tell_joke").target_profile("llama3")
greet = palang.import_program("hello::greet").target_profile("llama3")

translate = palang.import_program("hello::translate").target_profile("llama3")

def main():
    greeting = greet()
    print(greeting)

    joke = tell_joke.cast_as(JsonList)()
    print(joke)

    print(translate("Hi there", "French"))

if __name__ == "__main__":
    main()
